# -*- coding: utf-8 -*-

from zope.interface import Interface

class ILiveChat(Interface):
    """
    Marker interface for LiveChat
    """

class IMessageKeeper(Interface):
	"""
	Message keeper utility interface
	"""
	def login(chatroom,user):
		"""
		"""
	
	def logout(chatroom,user):
		"""
		"""
	
	def sendMessage(chatroom,user,message):
		"""
		"""
		
	def update(chatroom,user):
		"""
		"""
		
		


from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *
from zope.interface import implements
from qi.LiveChat.interfaces import ILiveChat
from Products.ATContentTypes.content.schemata import finalizeATCTSchema, ATContentTypeSchema
from Products.ATContentTypes.content.base import ATCTContent
from qi.LiveChat.config import PROJECTNAME
schema = Schema((
),
)

LiveChat_schema = ATContentTypeSchema.copy() + \
    schema.copy()

class LiveChat(ATCTContent):
    """
    """
    implements(ILiveChat)
    security = ClassSecurityInfo()
    schema = LiveChat_schema

    def __init__(self, *args, **kwargs):
        """
        """
        ATCTContent.__init__(self,*args, **kwargs)

registerType(LiveChat, PROJECTNAME)



from zope.interface import implements
from zope.component import getMultiAdapter
from Products.Five import BrowserView
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from qi.LiveChat.browser.ILiveChatView import ILiveChatView

class LiveChatView(BrowserView):
	"""
	"""
	implements(ILiveChatView)
	__call__ = ViewPageTemplateFile('livechat_view.pt')
	
	def __init__(self,context,request):
		BrowserView.__init__(self,context,request)
		self.mt = getToolByName(context,"portal_membership")

	def getUser(self):
		"""
		"""
		return self.mt.getAuthenticatedMember()
	
	def isAnonymous(self):
		"""
		"""
		portal_state = getMultiAdapter((self.context, self.request), name=u"plone_portal_state")
		if portal_state.anonymous():
			return True
		return False


#! /usr/bin/env python

# Twisted Imports
from twisted.internet import reactor
from twisted.web import xmlrpc, server
from twisted.web.xmlrpc import XMLRPC
from time import time
from copy import copy
from exceptions import KeyError
CLEANUP_INTERVAL = 10.0

class ChatRoom(object):
	"""Represents a chatroom
	""" 
	
	def __init__(self):
		"""
		"""
		self.aliveUsers = set()
		self.lastAccess = dict() 
		self.messages = dict() 
		self.lastUserChange = time()
		
	def login(self,userID,now):
		"""
		"""
		self.lastUserChange = now
		self.lastAccess[userID] = now
		self.aliveUsers.add(userID)
		messTss = self.messages.keys()
		messTss.sort()
		messages=[self.messages[ts] for ts in messTss]
		return [list(self.aliveUsers),messages]
			
	def logout(self,userID,now):
		"""
		"""
		try:
			del self.lastAccess[userID]
		finally:
			self.aliveUsers.remove(userID)
			self.lastUserChange = now

		
	def cleanup(self,now,cleanupInterval):
		"""
		"""
		
		messKeys = copy(self.messages.keys())
		for tstamp in messKeys:
			if (now-tstamp)>=cleanupInterval:
				del self.messages[tstamp]
		users  = copy(self.lastAccess.keys())
		for user in users:
			if (now-self.lastAccess[user])>=cleanupInterval:
				del self.lastAccess[user]
				self.aliveUsers.remove(user)
				self.lastUserChange = now
				
	def update(self,userID,now):
		"""
		"""
		
		users = []
		try:
			if self.lastUserChange>self.lastAccess[userID]:
				users = list(self.aliveUsers)
		except KeyError:
			return self.login(userID,now)
		tss=[ts for ts in self.messages.keys() if ts>self.lastAccess[userID]]
		tss.sort()
		messages = [self.messages[ts] for ts in tss]
		self.lastAccess[userID] = now
		return [users,messages]

	def sendMessage(self,userID,message,now):
		"""
		"""
		self.messages[now] = message
		return self.update(userID,now)
			

class MessageKeeper(XMLRPC):
	"""
	"""

	def __init__(self):
		XMLRPC.__init__(self)
		self.chatrooms = dict() 
		reactor.callLater(CLEANUP_INTERVAL, self.cleanup)

	def cleanup(self):
		"""
		"""
		now = time()
		for room in self.chatrooms:
			self.chatrooms[room].cleanup(now,CLEANUP_INTERVAL)	
		reactor.callLater(CLEANUP_INTERVAL, self.cleanup)
		

	def xmlrpc_login(self,chatUID,userID):
		"""
		"""
		now = time()
		try:
			users = self.chatrooms[chatUID].login(userID,now)
		except KeyError:
			self.chatrooms[chatUID] = ChatRoom()
			users = self.chatrooms[chatUID].login(userID,now)
		return users

	def xmlrpc_update(self,chatUID,userID):
		"""
		"""
		now = time()
		try:
			return self.chatrooms[chatUID].update(userID,now)
		except KeyError:
			self.chatrooms[chatUID] = ChatRoom()
			return self.chatrooms[chatUID].update(userID,now)
		
	def xmlrpc_sendMessage(self,chatUID,userID,message):
		now = time()
		return self.chatrooms[chatUID].sendMessage(userID,message,now)
			
		
	def xmlrpc_logout(self,chatUID,userID):
		"""
		"""
		now = time()
		self.chatrooms[chatUID].logout(userID,now)
		return ""
				
if __name__ == '__main__':
	
	theKeeper = MessageKeeper()
	reactor.listenTCP(8000, server.Site(theKeeper))
	reactor.run() 



from zope.component import getUtility
from kss.core import KSSView
from DateTime import DateTime
from elementtree.ElementTree import XML as parseET
from Products.CMFCore.utils import getToolByName 
from Products.CMFDefault.utils import scrubHTML
from emoticons	import replaceEmoticons
from plone.intelligenttext.transforms import convertHtmlToWebIntelligentPlainText, convertWebIntelligentPlainTextToHtml
from qi.LiveChat.interfaces import IMessageKeeper

from qi.LiveChat import LiveChatMessageFactory as _
class LiveChatKSS(KSSView):
	"""
	"""
	def __init__(self, *args):
		KSSView.__init__(self, *args)
		self.chat = self.context.aq_inner
		self.core = self.getCommandSet('core')
		self.lckss = self.getCommandSet('livechat')
		self.plone = self.getCommandSet('plone')
		self.mt = getToolByName(self.chat,"portal_membership")
		self.mk = getUtility(IMessageKeeper)
		self.uid = self.chat.UID()

	def login(self):
		"""
		A user connects to the chat room.
		"""
	
		user = str(self.mt.getAuthenticatedMember())
		try:
			(users,messages) = self.mk.login(self.uid,user)
		except:
			return self._connectionError()
		self._updateUsers(users)
		self._updateMessages(messages)
		self.core.focus("#msgInput")
		return self.render()
	
	def logout(self):
		"""
		User log outs from the chat
		"""
		user = str(self.mt.getAuthenticatedMember())
		self.mk.logout(self.uid,user)	
		return self.render()
	
	def ping(self,message=None):
		"""
		"""
		user = str(self.mt.getAuthenticatedMember())

		if message:
			dt=DateTime()
			message = convertHtmlToWebIntelligentPlainText(message)
			message = convertWebIntelligentPlainTextToHtml(message)
			message = replaceEmoticons(message)
			time = dt.Time()
			try:
				(users,messages) = self.mk.sendMessage(self.uid,user,[time,user,message])
			except:
				return self._connectionError()
			self._updateUsers(users)
			self._updateMessages(messages)
						
			# The following is done because KSS setAttribute does not work on <input> value. 
			#self.core.setAttribute(self.core.getHtmlIdSelector('#msgInput'),'value','')
			self.lckss.resetInput("#msgInput")
			self.core.focus("#msgInput")
			return self.render()
		try:
			[users,messages] = self.mk.update(self.uid,user)
		except:
			return self._connectionError()
		
		self._updateUsers(users)
		self._updateMessages(messages)
		self.plone.issuePortalMessage(_(u""))
		return self.render()
	
	def _connectionError(self):
		"""
		"""
		self.plone.issuePortalMessage(_(u"Error connecting to Live Chat server."),'error')
		return self.render()
	
	def _updateMessages(self,messages):
		"""
		"""
		if messages:
			messagesElem = self.core.getHtmlIdSelector("messages")
			for (time,user,message)	 in messages:
				elem = '<div><span class="msg_time">%s</span><span class="msg_user">%s</span><span>%s</span></div>'%(time,user,message)
				self.core.insertHTMLAsLastChild(messagesElem, elem)
			self.lckss.resetScrollbar(messagesElem)
			
	
	def _updateUsers(self,users):
		"""
		"""
		if users:
			users.sort()
			content = ""
			for user in users:
				content=content+"<div>%s</div>"%user
			self.core.replaceInnerHTML('#loggedUsers', content)


from qi.LiveChat.kss.ILiveChatKSSCommands import ILiveChatKSSCommands
from kss.core import CommandSet
from zope import interface

class LiveChatKSSCommands(CommandSet):
	interface.implements(ILiveChatKSSCommands)
	
	def resetScrollbar(self, selector):
		command = self.commands.addCommand('resetScrollbar', selector)
	def resetInput(self, selector):
		command = self.commands.addCommand('resetInput', selector)



from zope.interface import implements
from zope.component import getUtility
from zope.app.component.hooks import getSite
from qi.LiveChat.interfaces import IMessageKeeper
from xmlrpclib import Server

class XMLRPCMessageKeeper(object):
	"""
	"""
	implements(IMessageKeeper)
	def __init__(self):
		"""
		"""
		self.rpcserver = Server('http://localhost:8000')		
		
	def login(self,chatUID,userID):
		"""
		"""
		return self.rpcserver.login(chatUID,userID)
			
	def logout(self,chatUID,userID):
		"""
		"""
		self.rpcserver.logout(chatUID,userID)
		
	def sendMessage(self,chatUID,userID,message):
		"""
		"""
		return self.rpcserver.sendMessage(chatUID,userID,message)

	def update(self,chatUID,userID):
		"""
		"""
		return self.rpcserver.update(chatUID,userID)
		


from config import *
from zope.i18nmessageid import MessageFactory
from Products.CMFCore import utils
from Products.Archetypes import atapi

LiveChatMessageFactory = MessageFactory('qi.LiveChat')

def initialize(context):
	import content.LiveChat

	content_types, constructors, ftis = atapi.process_types(
		atapi.listTypes(config.PROJECTNAME),
		config.PROJECTNAME)

	for atype, constructor in zip(content_types, constructors):
		utils.ContentInit("%s: %s" % (config.PROJECTNAME, atype.portal_type),
			content_types	   = (atype,),
			permission		   = config.ADD_PERMISSIONS[atype.portal_type],
			extra_constructors = (constructor,),
			).initialize(context)
